"""
__init__.py: arenarobot scripts module root.

Created by Perry Naseck on 01/25/212

Copyright (c) 2022, The CONIX Research Center
All rights reserved.

This source code is licensed under the BSD-3-Clause license found in the
LICENSE file in the root directory of this source tree.
"""
