"""
__init__.py: arenavideocall scripts module root.

Created by Perry Naseck on 11/3/21.

Copyright (c) 2021, The CONIX Research Center
All rights reserved.

This source code is licensed under the BSD-3-Clause license found in the
LICENSE file in the root directory of this source tree.
"""
