"""
const.py: Constants for the occupancy grid.

Created by Perry Naseck on 6/29/21.

Copyright (c) 2021, The CONIX Research Center
All rights reserved.

This source code is licensed under the BSD-3-Clause license found in the
LICENSE file in the root directory of this source tree.
"""

UPDATE_ACTION = 'action'
UPDATE_LOC = 'loc'
UPDATE_OCCUPANT = 'occupant'

UPDATE_ACTION_OCCUPY = 'occupy'
UPDATE_ACTION_LEAVE = 'leave'
