"""Package version information."""
# coding: utf-8
# file generated by setuptools_scm
# don't change, don't track in version control
# flake8: noqa
version = '2.2.0'  # pylint: disable=invalid-name
version_tuple = (2, 2, 0)  # pylint: disable=invalid-name
