"""
transform.py: Definitions for LiCosa packet transformer.

Created by Perry Naseck on 2/24/22.

Copyright (c) 2022, The CONIX Research Center
All rights reserved.

This source code is licensed under the BSD-3-Clause license found in the
LICENSE file in the root directory of this source tree.
"""


class LiCosaPacketTransformer():
    """LiCosa packet transformer."""
